@@ -70,6 +70,14 @@ def s3_base():
     import shlex
     import subprocess
 
+    try:
+        # should fail since we didn't start server yet
+        r = requests.get(endpoint_uri)
+    except:
+        pass
+    else:
+        if r.ok:
+            raise RuntimeError("moto server already up")
     if "AWS_SECRET_ACCESS_KEY" not in os.environ:
         os.environ["AWS_SECRET_ACCESS_KEY"] = "foo"
     if "AWS_ACCESS_KEY_ID" not in os.environ:
@@ -1785,6 +1793,18 @@ def test_async_s3(s3):
     asyncio.run(_())
 
 
+def test_cat_ranges(s3):
+    data = b"a string to select from"
+    fn = test_bucket_name + "/parts"
+    s3.pipe(fn, data)
+
+    assert s3.cat_file(fn) == data
+    assert s3.cat_file(fn, start=5) == data[5:]
+    assert s3.cat_file(fn, end=5) == data[:5]
+    assert s3.cat_file(fn, start=1, end=-1) == data[1:-1]
+    assert s3.cat_file(fn, start=-5) == data[-5:]
+
+
 @pytest.mark.skipif(sys.version_info < (3, 7), reason="no asyncio.run in py36")
 def test_async_s3_old(s3):
     async def _():
@@ -2099,3 +2119,26 @@ def test_copy_file_without_etag(s3, monkeypatch):
 
     s3.cp_file(file["name"], test_bucket_name + "/copy_tests/file2")
     assert s3.info(test_bucket_name + "/copy_tests/file2")["ETag"] is not None
+
+
+def test_find_with_prefix(s3):
+    for cursor in range(100):
+        s3.touch(test_bucket_name + f"/prefixes/test_{cursor}")
+
+    s3.touch(test_bucket_name + "/prefixes2")
+    assert len(s3.find(test_bucket_name + "/prefixes")) == 100
+    assert len(s3.find(test_bucket_name, prefix="prefixes")) == 101
+
+    assert len(s3.find(test_bucket_name + "/prefixes/test_")) == 0
+    assert len(s3.find(test_bucket_name + "/prefixes", prefix="test_")) == 100
+    assert len(s3.find(test_bucket_name + "/prefixes/", prefix="test_")) == 100
+
+    test_1s = s3.find(test_bucket_name + "/prefixes/test_1")
+    assert len(test_1s) == 1
+    assert test_1s[0] == test_bucket_name + "/prefixes/test_1"
+
+    test_1s = s3.find(test_bucket_name + "/prefixes/", prefix="test_1")
+    assert len(test_1s) == 11
+    assert test_1s == [test_bucket_name + "/prefixes/test_1"] + [
+        test_bucket_name + f"/prefixes/test_{cursor}" for cursor in range(10, 20)
+    ]
